#define PGP_WIN32
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "PgpSdk1.h"
#include "PGPBuffers.h"
#include "PGPfiles.h"

TPGPsdk *PGPsdk=NULL;
HINSTANCE hPgpSdk=NULL;
#define mb(txt) MessageBox(owner->Handle,txt,"TEST",MB_OK);
//---------------------------------------------------------------------------
bool __fastcall HasSdk()
{
   return hPgpSdk!=NULL;
}
//---------------------------------------------------------------------------
__fastcall TPGPsdk::TPGPsdk(TForm *Owner, HINSTANCE &hPgpSdk)
{
   owner = Owner;
   hSdkLib = hPgpSdk;

   err=PGPsdkInit();
   if (IsPgpError(err,"Unable to initialize PGPsdk!"))
      return;

   err=PGPGetSDKVersion(&sdkversion);
   if (IsPgpError(err,"Failed to establish SDK version!"))
      return;

   err=PGPNewContext(kPGPsdkAPIVersion,&context);
   if (IsPgpError(err,"Failed to initialize PGP context structure!"))
      return; // false;

   err=PGPsdkLoadDefaultPrefs(context);
   if (IsPgpError(err,"Unable to load PGPsdk default preferences!"))
      return; // false;

   PGPFileSpecRef pubSpec = kInvalidPGPFileSpecRef;
   PGPFileSpecRef secSpec = kInvalidPGPFileSpecRef;

   // if first attempt fails, use prefSelector from earlier sdk version
   err=PGPsdkPrefGetFileSpec(context,(PGPsdkPrefSelector)5,&pubSpec);
   if (err==-12000) err=PGPsdkPrefGetFileSpec(context,(PGPsdkPrefSelector)0,&pubSpec);
   if (IsPgpError(err,"Unable to get PGP public keyring file spec!"))
      goto exit;

   err=PGPGetFullPathFromFileSpec(pubSpec,&pubringpkr);
   if (IsPgpError(err,"Unable to get PGP public keyring path!"))
      goto exit;

   // if first attempt fails, use prefSelector from earlier sdk version
   err=PGPsdkPrefGetFileSpec(context,(PGPsdkPrefSelector)6,&secSpec);
   if (err==-12000) err=PGPsdkPrefGetFileSpec(context,(PGPsdkPrefSelector)1,&secSpec);
   if (IsPgpError(err,"Unable to get PGP secret keyring file spec!"))
      goto exit;

   err=PGPGetFullPathFromFileSpec(secSpec,&secringskr);
   if (IsPgpError(err,"Unable to get PGP secret keyring path!"))
      goto exit;

   err=PGPGetSDKVersion(&apiversion);
   if (IsPgpError(err,"Failed to get PGPsdk API version!"))
      goto exit;

   StrLower(pubringpkr);
   StrLower(secringskr);
exit:
   if (PGPFileSpecRefIsValid(secSpec)) PGPFreeFileSpec(secSpec);
   if (PGPFileSpecRefIsValid(pubSpec)) PGPFreeFileSpec(pubSpec);
}
//---------------------------------------------------------------------------
__fastcall TPGPsdk::~TPGPsdk()
{
   err = PGPsdkCleanup();
   IsPgpError(err,"Unable to cleanup PGPsdk for exit!");
}
//---------------------------------------------------------------------------
bool __fastcall TPGPsdk::ExportDefaultKeys(TStrings *exports)
{
   exports->Clear();
   TStringList *revokes = new TStringList();
   TStringList *disables = new TStringList();
   PGPKeySetRef keyset = kInvalidPGPKeySetRef;

   err=PGPOpenDefaultKeyRings(context,false,&keyset);
   if (IsPgpError(err,"Failed to open default keyrings!"))
      goto exit;

   err=GetRevokedKeyIds(keyset,revokes);
   if (IsPGPError(err))
      goto exit;

   err=GetDisableKeyIds(keyset,disables);
   if (IsPGPError(err))
      goto exit;

   err=ExportKeySet(keyset,kPGPExportFormat_Basic,true,exports); // true = include privates
   if (IsPGPError(err))
      goto exit;

   if (!disables->Text.IsEmpty()) exports->Insert(0,"Disabled: "+disables->CommaText);
   if (!revokes->Text.IsEmpty())exports->Insert(0,"Revoked: "+revokes->CommaText);

exit:
   if (PGPKeySetRefIsValid(keyset)) PGPFreeKeySet(keyset);
   delete disables;
   delete revokes;
   return IsntPGPError(err);
}
//---------------------------------------------------------------------------
bool __fastcall TPGPsdk::ExportDefaultKeysFast(TStrings *exports)
{
   exports->Clear();
   PGPKeySetRef keyset = kInvalidPGPKeySetRef;
   PGPKeySetRef nodisables = kInvalidPGPKeySetRef;
   PGPFilterRef disfilter = kInvalidPGPFilterRef;
   PGPKeySetRef norevokes = kInvalidPGPKeySetRef;
   PGPFilterRef revfilter = kInvalidPGPFilterRef;

   err=PGPOpenDefaultKeyRings(context,false,&keyset);
   if (IsPgpError(err,"Failed to open default keyrings!"))
      goto exit;

   err = PGPNewKeyRevokedFilter(context,false,&revfilter);
   if (IsPgpError(err,"Failed to create revoked key filter."))
      goto exit;

   err = PGPFilterKeySet(keyset,revfilter,&norevokes);
   if (IsPgpError(err,"Failed to get keys from revoked filter."))
      goto exit;

   err = PGPNewKeyDisabledFilter(context,false,&disfilter);
   if (IsPgpError(err,"Failed to create disabled key filter."))
      goto exit;

   err = PGPFilterKeySet(norevokes,disfilter,&nodisables);
   if (IsPgpError(err,"Failed to get keys from disabled filter."))
      goto exit;

   err=ExportKeySet(nodisables,kPGPExportFormat_Basic,true,exports);
   if (IsPGPError(err))                           // true = include privates
      goto exit;

exit:
   if (PGPFilterRefIsValid(revfilter)) PGPFreeFilter(revfilter);
   if (PGPKeySetRefIsValid(norevokes)) PGPFreeKeySet(norevokes);
   if (PGPFilterRefIsValid(disfilter)) PGPFreeFilter(disfilter);
   if (PGPKeySetRefIsValid(nodisables)) PGPFreeKeySet(nodisables);
   if (PGPKeySetRefIsValid(keyset)) PGPFreeKeySet(keyset);
   return IsntPGPError(err);
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::ExportKeySet(PGPKeySetRef keys, PGPExportFormat &ExportFormat,
   bool IncludePrivates, TStrings *exports)
{
   exports->Clear();
   void *data=NULL; PGPSize size;

   err=PGPExportKeySet(keys,
      PGPOAllocatedOutputBuffer(context,&data,-1,&size), // -1 = no size constraint
      PGPOExportFormat(context,ExportFormat),
      PGPOExportPrivateKeys(context,IncludePrivates),
      PGPOVersionString(context,"N/A"),
      PGPOLastOption(context)
   );
   if (IsPgpError(err,"Failed to export keys!"))
      goto exit;

   exports->SetText((char*)data);
exit:
   if (data) PGPFreeData(data);
   return err;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::GetDisableKeyIds(PGPKeySetRef keyset, TStrings *keyids)
{
   keyids->Clear();
   PGPKeyRef key = kInvalidPGPKeyRef;
   PGPFilterRef filter = kInvalidPGPFilterRef;
   PGPKeySetRef diskeys = kInvalidPGPKeySetRef;
   PGPKeyListRef diskeyslist = kInvalidPGPKeyListRef;
   PGPKeyIterRef listIterator = kInvalidPGPKeyIterRef;
   char keyidstr[kPGPMaxKeyIDStringSize];
   PGPKeyID keyid;

   err = PGPNewKeyDisabledFilter(context,true,&filter);
   if (IsPgpError(err,"Failed to create disabled key filter."))
      goto exit;

   err = PGPFilterKeySet(keyset,filter,&diskeys);
   if (IsPgpError(err,"Failed to get disabled keys."))
      goto exit;

   err = PGPOrderKeySet(diskeys,kPGPAnyOrdering,&diskeyslist);
   if (IsPgpError(err,"Failed to create disabled key list."))
      goto exit;

   err=PGPNewKeyIter(diskeyslist,&listIterator);
   if (IsPgpError(err,"Failed to create key list iterator."))
      goto exit;

   while (IsntPGPError(err=PGPKeyIterNext(listIterator,&key)) && key)
   {
      err=PGPGetKeyIDFromKey(key,&keyid);
      if (IsPgpError(err,"Failed to get key id from revoked key."))
         goto exit;

      err=PGPGetKeyIDString(&keyid,kPGPKeyIDString_Abbreviated,keyidstr);
      if (IsPgpError(err,"Failed to get key id string!"))
         goto exit;

      keyids->Add(keyidstr);
   }
   if (err==kPGPError_EndOfIteration) err = kPGPError_NoErr;
   if (IsPgpError(err,"Failed to complete iteration of revoked key list."))
      goto exit;

exit:
   if (PGPKeyIterRefIsValid(listIterator)) PGPFreeKeyIter(listIterator);
   if (PGPKeyListRefIsValid(diskeyslist)) PGPFreeKeyList(diskeyslist);
   if (PGPKeySetRefIsValid(diskeys)) PGPFreeKeySet(diskeys);
   if (PGPFilterRefIsValid(filter)) PGPFreeFilter(filter);
   return err;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::GetRevokedKeyIds(PGPKeySetRef keyset, TStrings *keyids)
{
   keyids->Clear();
   PGPKeyRef key = kInvalidPGPKeyRef;
   PGPFilterRef filter = kInvalidPGPFilterRef;
   PGPKeySetRef revkeys = kInvalidPGPKeySetRef;
   PGPKeyListRef revkeyslist = kInvalidPGPKeyListRef;
   PGPKeyIterRef listIterator = kInvalidPGPKeyIterRef;
   char keyidstr[kPGPMaxKeyIDStringSize];
   PGPKeyID keyid;

   err = PGPNewKeyRevokedFilter(context,true,&filter);
   if (IsPgpError(err,"Failed to create revoked key filter."))
      goto exit;

   err = PGPFilterKeySet(keyset,filter,&revkeys);
   if (IsPgpError(err,"Failed to get revoked keys."))
      goto exit;

   err = PGPOrderKeySet(revkeys,kPGPAnyOrdering,&revkeyslist);
   if (IsPgpError(err,"Failed to create revoked key list."))
      goto exit;

   err=PGPNewKeyIter(revkeyslist, &listIterator);
   if (IsPgpError(err,"Failed to create key list iterator."))
      goto exit;

   while (IsntPGPError(err=PGPKeyIterNext(listIterator,&key)) && key)
   {
      err=PGPGetKeyIDFromKey(key,&keyid);
      if (IsPgpError(err,"Failed to get key id from revoked key."))
         goto exit;

      err=PGPGetKeyIDString(&keyid,kPGPKeyIDString_Abbreviated,keyidstr);
      if (IsPgpError(err,"Failed to get key id string!"))
         goto exit;

      keyids->Add(keyidstr);
   }
   if (err==kPGPError_EndOfIteration) err = kPGPError_NoErr;
   if (IsPgpError(err,"Failed to complete iteration of revoked key list."))
      goto exit;

exit:
   if (PGPKeyIterRefIsValid(listIterator)) PGPFreeKeyIter(listIterator);
   if (PGPKeyListRefIsValid(revkeyslist)) PGPFreeKeyList(revkeyslist);
   if (PGPKeySetRefIsValid(revkeys)) PGPFreeKeySet(revkeys);
   if (PGPFilterRefIsValid(filter)) PGPFreeFilter(filter);
   return err;
}
//---------------------------------------------------------------------------
void __fastcall TPGPsdk::ShowPGPError(PGPError err, AnsiString errmsg)
{
   if(IsPGPError(err))
   {
  	  char *buf = new char[256];
 	  PGPGetErrorString(err, 255, buf);
      MessageBox(owner->Handle,
         AnsiString(errmsg+"\nError: ("+AnsiString(err)+") "+AnsiString(buf)).c_str(),
        "PGP Error",
         MB_OK
      );
      delete buf;
   }
}
//---------------------------------------------------------------------------
bool __fastcall TPGPsdk::CheckPGP6NeedsUpdate(TControl *ForHourGlass)
{
   if (!PGP6AUTOUPDATE) return false;

   int QDKpub = FileAge(pubringpkr); // QDK = sdk keyrings
   int PGPpub = FileAge(PgpPublicRing());    // quicksilver keyrings
   int QDKsec = FileAge(secringskr);
   int PGPsec = FileAge(PgpSecretRing());
   if (QDKpub<0 || QDKsec<0) return false;   // -1 returned if file does not exist
   if (PGPpub<0 || PGPsec<0 || QDKpub>PGPpub || QDKsec>PGPsec)
   {
      UpdateQSKeyrings(ForHourGlass);
      return true;
   }
   return false;
}
//---------------------------------------------------------------------------
bool __fastcall TPGPsdk::UpdateQSKeyrings(TControl *ForHourGlass)
{
   if (USEPGPKEYSEXE) return UpdateQSKeyringsFast(ForHourGlass);
   else return UpdateQSKeyringsSlow(ForHourGlass);
}
//---------------------------------------------------------------------------
bool __fastcall TPGPsdk::UpdateQSKeyringsSlow(TControl *ForHourGlass)
{
   AppMainForm->Perform(WM_SHOWSTATUS,0,(LPARAM)"Updating...");
   if (ForHourGlass) ForHourGlass->Cursor = crHourGlass;
   TStringList *block = new TStringList();
   ExportDefaultKeys(block);
   if (!block->Count)
   {
      if (ForHourGlass) ForHourGlass->Cursor = crDefault;
      MsgBoxHwnd(owner->Handle,"Failed to import keys!","File Error",MB_OK);
      return false;
   }

   AnsiString line,rids,dids,id;
   while (block->Count)
   {
      line = block->Strings[0];
      if (line.Pos("-----BEGIN PGP")==1) break;
      if (line.Pos("Disabled: ")==1) dids = line.SubString(11,1024);
      if (line.Pos("Revoked: " )==1) rids = line.SubString(10,1024);
      block->Delete(0);
   }
   if (block->Strings[0].Pos("-----BEGIN PGP")==1)
   {
      TPGPbuffer *secrets,*publics,*newkeys,*pubs,*secs,*key,*chk,*pkt,*pub;

      pkt = new TPGPbuffer();
      pub = new TPGPbuffer();
      key = new TPGPbuffer();
      chk = new TPGPbuffer();
      secrets = new TPGPbuffer();
      publics = new TPGPbuffer();
      newkeys = new TPGPbuffer(block);
      // dispurse to publics and secrets
      while (key->copykeyfrom(newkeys,REWIND))
      {
         if (key->ispublic)
         {
            // add undefined trust packets back to keys
            while (pkt->copypacketfrom(key,REWIND))
            {
               pub->copyfrom(pkt);
               byte keytrust[3] = { 176,1,0 };
               byte sigtrust[3] = { 176,1,0 };
               byte uidtrust[3] = { 176,1,0 };
               if (PublicKeyPacket(pkt->CTB)) pub->Write(keytrust,3); else
               if (SignaturePacket(pkt->CTB)) pub->Write(sigtrust,3); else
               if (UserIdPacket(pkt->CTB)) pub->Write(uidtrust,3);
               pkt->Clear();
            }
            if (dids.Pos("0x"+pub->idtext8)>=1) pub->disable();
            if (rids.Pos("0x"+pub->idtext8)<1)
                publics->copyfrom(pub);
            pub->clear();
         }
         else
            secrets->copyfrom(key);
         key->clear();
      }
      // load publics and add to new keys if not already present
      TStringList *uids = new TStringList();
      uids->Duplicates = dupIgnore;
      publics->getuserids(uids);
      pubs = new TPGPbuffer(PgpPublicRing());
      while (key->copykeyfrom(pubs,REWIND))
      {  // this actions ends with unmatched current keys copied to publics which
         if (uids->IndexOf(key->idtext8)>=0) // becomes the new keyring
            publics->copyfrom(key);
         key->clear();
      }
      // load secrets and add to new keys if not already present
      uids->Text = secrets->getuserids();
      secs = new TPGPbuffer(PgpSecretRing());
      while (key->copykeyfrom(secs,REWIND))
      {
         if (uids->IndexOf(key->idtext8)>=0)
            secrets->copyfrom(key);
         key->clear();
      }
      // save new keyrings to file
      publics->SaveToFile(PgpPublicRing());
      secrets->SaveToFile(PgpSecretRing());

      delete secs;
      delete pubs;
      delete key;
      delete chk;
      delete pub;
      delete pkt;
   }
   delete block;
   //reverse earlier visual cues
   if (ForHourGlass) ForHourGlass->Cursor = crDefault;
   //if (Owner) Owner->Caption = cap;
   AppMainForm->Perform(WM_SHOWSTATUS,0,(LPARAM)"");
   return true;
}
//---------------------------------------------------------------------------
bool __fastcall TPGPsdk::UpdateQSKeyringsFast(TControl *ForHourGlass)
{
   AppMainForm->Perform(WM_SHOWSTATUS,0,(LPARAM)"Updating...");
   if (ForHourGlass) ForHourGlass->Cursor = crHourGlass;

   TStringList *blocks = new TStringList();
   ExportDefaultKeysFast(blocks);
   if (!blocks->Count && IsPGPError(err))
   {
      if (ForHourGlass) ForHourGlass->Cursor = crDefault;
      MsgBoxHwnd(owner->Handle,"Failed to import keys!","File Error",MB_OK);
      delete blocks;
      return false;
   }

   TPGPbuffer *key = new TPGPbuffer();
   TPGPbuffer *secrets = new TPGPbuffer();
   TPGPbuffer *publics = new TPGPbuffer();
   TPGPbuffer *newkeys = new TPGPbuffer(blocks);
   while (key->copykeyfrom(newkeys))
   {
         if (key->ispublic) publics->copyfrom(key);
         else secrets->copyfrom(key);
         key->clear();
   }
   publics->SaveToFile(PgpPublicRing());
   secrets->SaveToFile(PgpSecretRing());
   delete newkeys;
   delete secrets;
   delete publics;
   delete key;
   delete blocks;

   if (ForHourGlass) ForHourGlass->Cursor = crDefault;
   AppMainForm->Perform(WM_SHOWSTATUS,0,(LPARAM)"");
   return true;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPGetErrorString(PGPError theError, PGPSize bufferSize, char *theString)
{
   PGPError ret=-1; PGPGETERRORSTRING errstring;
   if ((errstring=(PGPGETERRORSTRING)GetProcAddress(hSdkLib,"PGPGetErrorString"))!=NULL)
      ret = (errstring)(theError, bufferSize, theString);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPsdkInit()
{
   PGPError ret=-64; PGPSDKINIT pgpsdkinit;
   if ((pgpsdkinit=(PGPSDKINIT)GetProcAddress(hSdkLib,"PGPsdkInit"))!=NULL)
      ret = (pgpsdkinit)();
   return ret;
}
//---------------------------------------------------------------------------
bool __fastcall TPGPsdk::IsPgpError(PGPError err, char *errmsg)
{
   if (IsPGPError(err))
   {
      ShowPGPError(err,errmsg);
      return true;
   }
   return false;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPsdkCleanup()
{
   PGPError ret=-64; PGPSDKCLEANUP pgpsdkcleanup;
   if ((pgpsdkcleanup=(PGPSDKCLEANUP)GetProcAddress(hSdkLib,"PGPsdkCleanup"))!=NULL)
      ret = (pgpsdkcleanup)();
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPNewContext(PGPUInt32 clientAPIVersion, PGPContextRef *newContext)
{
   PGPError ret=-64; PGPNEWCONTEXT pgpnewcontext;
   if ((pgpnewcontext=(PGPNEWCONTEXT)GetProcAddress(hSdkLib,"PGPNewContext"))!=NULL)
      ret = (pgpnewcontext)(clientAPIVersion, newContext);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPsdkLoadDefaultPrefs(PGPContextRef context)
{
   PGPError ret=-64; PGPSDKLOADDEFAULTPREFS loadprefs;
   if ((loadprefs=(PGPSDKLOADDEFAULTPREFS)GetProcAddress(hSdkLib,"PGPsdkLoadDefaultPrefs"))!=NULL)
      ret = (loadprefs)(context);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPsdkPrefGetFileSpec(PGPContextRef context, PGPsdkPrefSelector selector, PGPFileSpecRef * outRef)
{
   PGPError ret=-64; PGPSDKPREFGETFILESPEC filespec;
   if ((filespec=(PGPSDKPREFGETFILESPEC)GetProcAddress(hSdkLib,"PGPsdkPrefGetFileSpec"))!=NULL)
      ret = (filespec)(context, selector, outRef);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPGetFullPathFromFileSpec(PGPFileSpecRef fileRef, char **fullPathPtr)
{
   PGPError ret=-64; PGPGETFULLPATHFROMFILESPEC fullpath;
   if ((fullpath=(PGPGETFULLPATHFROMFILESPEC)GetProcAddress(hSdkLib,"PGPGetFullPathFromFileSpec"))!=NULL)
      ret = (fullpath)(fileRef, fullPathPtr);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPGetSDKVersion(PGPUInt32 *version)
{
   PGPError ret=-64; PGPGETSDKVERSION sdkversion;
   if ((sdkversion=(PGPGETSDKVERSION)GetProcAddress(hSdkLib,"PGPGetSDKVersion"))!=NULL)
      ret = (sdkversion)(version);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPFreeFileSpec(PGPFileSpecRef fileRef)
{
   PGPError ret=-64; PGPFREEFILESPEC freespec;
   if ((freespec=(PGPFREEFILESPEC)GetProcAddress(hSdkLib,"PGPFreeFileSpec"))!=NULL)
      ret = (freespec)(fileRef);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPExportKeySet(PGPKeySetRef keys, PGPOptionListRef firstOpt,
   PGPOptionListRef secondOpt, PGPOptionListRef thirdOpt, PGPOptionListRef forthOpt,
   PGPOptionListRef fifthOpt)
{
   PGPError ret=-64; PGPEXPORTKEYSET exportset;
   if ((exportset=(PGPEXPORTKEYSET)GetProcAddress(hSdkLib,"PGPExportKeySet"))!=NULL)
      ret = (exportset)(keys, firstOpt, secondOpt, thirdOpt, forthOpt, fifthOpt);
   return ret;
}
//---------------------------------------------------------------------------
PGPOptionListRef __fastcall TPGPsdk::PGPOExportFormat(PGPContextRef context,
   PGPExportFormat exportFormat)
{
   PGPOptionListRef olref=(PGPOptionList*)-64; PGPOEXPORTFORMAT exportfmt;
   if ((exportfmt=(PGPOEXPORTFORMAT)GetProcAddress(hSdkLib,"PGPOExportFormat"))!=NULL)
      olref = (exportfmt)(context, exportFormat);
   return olref;
}
//---------------------------------------------------------------------------
PGPOptionListRef __fastcall TPGPsdk::PGPOExportPrivateKeys(PGPContextRef context,
   PGPBoolean exportKeys)
{
   PGPOptionListRef olref=(PGPOptionList*)-64; PGPOEXPORTPRIVATEKEYS exportprivs;
   if ((exportprivs=(PGPOEXPORTPRIVATEKEYS)GetProcAddress(hSdkLib,"PGPOExportPrivateKeys"))!=NULL)
      olref = (exportprivs)(context, exportKeys);
   return olref;
}
//---------------------------------------------------------------------------
PGPOptionListRef __fastcall TPGPsdk::PGPOVersionString(PGPContextRef context,
   char const *version)
{
   PGPOptionListRef olref=(PGPOptionList*)-64; PGPOVERSIONSTRING verstr;
   if ((verstr=(PGPOVERSIONSTRING)GetProcAddress(hSdkLib,"PGPOVersionString"))!=NULL)
      olref = (verstr)(context, version);
   return olref;
}
//---------------------------------------------------------------------------
PGPOptionListRef __fastcall TPGPsdk::PGPOLastOption(PGPContextRef context)
{
   PGPOptionListRef olref=(PGPOptionList*)-64; PGPOLASTOPTION lastopt;
   if ((lastopt=(PGPOLASTOPTION)GetProcAddress(hSdkLib,"PGPOLastOption"))!=NULL)
      olref = (lastopt)(context);
   return olref;
}
//---------------------------------------------------------------------------
PGPOptionListRef __fastcall TPGPsdk::PGPOAllocatedOutputBuffer(PGPContextRef context,
   void **buffer, PGPSize maximumBufferSize, PGPSize *actualBufferSize)
{
   PGPOptionListRef olref=(PGPOptionList*)-64; PGPOALLOCATEDOUTPUTBUFFER allbuf;
   if ((allbuf=(PGPOALLOCATEDOUTPUTBUFFER)GetProcAddress(hSdkLib,"PGPOAllocatedOutputBuffer"))!=NULL)
      olref = (allbuf)(context, buffer, maximumBufferSize, actualBufferSize);
   return olref;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPFreeKeySet(PGPKeySetRef keys)
{
   PGPError ret=-64; PGPFREEKEYSET freeset;
   if ((freeset=(PGPFREEKEYSET)GetProcAddress(hSdkLib,"PGPFreeKeySet"))!=NULL)
      ret = (freeset)(keys);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPFreeData(void *allocation)
{
   PGPError ret=-64; PGPFREEDATA freedat;
   if ((freedat=(PGPFREEDATA)GetProcAddress(hSdkLib,"PGPFreeData"))!=NULL)
      ret = (freedat)(allocation);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPOpenDefaultKeyRings(PGPContextRef context,
   PGPKeyRingOpenFlags openFlags, PGPKeySetRef *keySet)
{
   PGPError ret=-64; PGPOPENDEFAULTKEYRINGS opendefs;
   if ((opendefs=(PGPOPENDEFAULTKEYRINGS)GetProcAddress(hSdkLib,"PGPOpenDefaultKeyRings"))!=NULL)
      ret = (opendefs)(context, openFlags, keySet);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPNewKeyDisabledFilter(PGPContextRef context,
   PGPBoolean disabled, PGPFilterRef *outFilter)
{
   PGPError ret=-64; PGPNEWKEYDISABLEDFILTER filter;
   if ((filter=(PGPNEWKEYDISABLEDFILTER)GetProcAddress(hSdkLib,"PGPNewKeyDisabledFilter"))!=NULL)
      ret = (filter)(context, disabled, outFilter);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPNewKeyRevokedFilter(PGPContextRef context, PGPBoolean revoked,
      PGPFilterRef *outFilter)
{
   PGPError ret=-64; PGPNEWKEYREVOKEDFILTER filter;
   if ((filter=(PGPNEWKEYREVOKEDFILTER)GetProcAddress(hSdkLib,"PGPNewKeyRevokedFilter"))!=NULL)
      ret = (filter)(context, revoked, outFilter);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPFilterKeySet(PGPKeySetRef origSet, PGPFilterRef filter,
      PGPKeySetRef *resultSet)
{
   PGPError ret=-64; PGPFILTERKEYSET filterset;
   if ((filterset=(PGPFILTERKEYSET)GetProcAddress(hSdkLib,"PGPFilterKeySet"))!=NULL)
      ret = (filterset)(origSet, filter, resultSet);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPOrderKeySet(PGPKeySetRef src, PGPKeyOrdering order,
      PGPKeyListRef *outRef )
{
   PGPError ret=-64; PGPORDERKEYSET orderset;
   if ((orderset=(PGPORDERKEYSET)GetProcAddress(hSdkLib,"PGPOrderKeySet"))!=NULL)
      ret = (orderset)(src, order, outRef);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPNewKeyIter(PGPKeyListRef keys, PGPKeyIterRef *outRef)
{
   PGPError ret=-64; PGPNEWKEYITER iter;
   if ((iter=(PGPNEWKEYITER)GetProcAddress(hSdkLib,"PGPNewKeyIter"))!=NULL)
      ret = (iter)(keys, outRef);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPGetKeyIDFromKey(PGPKeyRef key, PGPKeyID *id)
{
   PGPError ret=-64; PGPGETKEYIDFROMKEY getid;
   if ((getid=(PGPGETKEYIDFROMKEY)GetProcAddress(hSdkLib,"PGPGetKeyIDFromKey"))!=NULL)
      ret = (getid)(key, id);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPGetKeyIDFromString(const char *string, PGPKeyID *id)
{
   PGPError ret=-64; PGPGETKEYIDFROMSTRING getid;
   if ((getid=(PGPGETKEYIDFROMSTRING)GetProcAddress(hSdkLib,"PGPGetKeyIDFromString"))!=NULL)
      ret = (getid)(string, id);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPGetKeyIDString(PGPKeyID const *ref, PGPKeyIDStringType type,
      char outString[kPGPMaxKeyIDStringSize])
{
   PGPError ret=-64; PGPGETKEYIDSTRING getstr;
   if ((getstr=(PGPGETKEYIDSTRING)GetProcAddress(hSdkLib,"PGPGetKeyIDString"))!=NULL)
      ret = (getstr)(ref, type, outString);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPKeyIterNext(PGPKeyIterRef iter, PGPKeyRef *outRef)
{
   PGPError ret=-64; PGPKEYITERNEXT iternext;
   if ((iternext=(PGPKEYITERNEXT)GetProcAddress(hSdkLib,"PGPKeyIterNext"))!=NULL)
      ret = (iternext)(iter, outRef);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPFreeKeyIter(PGPKeyIterRef iter)
{
   PGPError ret=-64; PGPFREEKEYITER freeiter;
   if ((freeiter=(PGPFREEKEYITER)GetProcAddress(hSdkLib,"PGPFreeKeyIter"))!=NULL)
      ret = (freeiter)(iter);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPFreeKeyList(PGPKeyListRef keys)
{
   PGPError ret=-64; PGPFREEKEYLIST freelist;
   if ((freelist=(PGPFREEKEYLIST)GetProcAddress(hSdkLib,"PGPFreeKeyList"))!=NULL)
      ret = (freelist)(keys);
   return ret;
}
//---------------------------------------------------------------------------
PGPError __fastcall TPGPsdk::PGPFreeFilter(PGPFilterRef filter)
{
   PGPError ret=-64; PGPFREEFILTER freefilter;
   if ((freefilter=(PGPFREEFILTER)GetProcAddress(hSdkLib,"PGPFreeFilter"))!=NULL)
      ret = (freefilter)(filter);
   return ret;
}
//---------------------------------------------------------------------------

